﻿#define DEBUG_AGENT
using System.Windows;
using Microsoft.Phone.Scheduler;

using System.Net;
using System;
using System.IO;
using System.Xml;
using System.ServiceModel.Syndication;
using System.Linq;
using Microsoft.Phone.Shell;

namespace FeedAgent
{
    public class ScheduledAgent : ScheduledTaskAgent
    {
        private static volatile bool _classInitialized;

        /// <remarks>
        /// ScheduledAgent コンストラクターは、UnhandledException ハンドラーを初期化します
        /// </remarks>
        public ScheduledAgent()
        {
            if (!_classInitialized)
            {
                _classInitialized = true;
                // マネージ例外ハンドラーをサブスクライブします
                Deployment.Current.Dispatcher.BeginInvoke(delegate
                {
                    Application.Current.UnhandledException += ScheduledAgent_UnhandledException;
                });
            }
        }

        /// ハンドルされない例外の発生時に実行されるコード
        private void ScheduledAgent_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // ハンドルされない例外が発生しました。デバッガーで中断します。
                System.Diagnostics.Debugger.Break();
            }
        }

        /// <summary>
        /// スケジュールされたタスクを実行するエージェント
        /// </summary>
        /// <param name="task">
        /// 呼び出されたタスク
        /// </param>
        /// <remarks>
        /// このメソッドは、定期的なタスクまたはリソースを集中的に使用するタスクの呼び出し時に呼び出されます
        /// </remarks>
        protected override void OnInvoke(ScheduledTask task)
        {
            Refresh();

            //TODO: バックグラウンドでタスクを実行するためのコードを追加します
#if DEBUG_AGENT
            //デバッグ実行中は60秒後に次のタスクが起動される
            ScheduledActionService.LaunchForTest(task.Name, TimeSpan.FromSeconds(60));
#endif
            //通常はここでバックグラウンド処理の終了を宣言するが
            //今回は非同期処理があるため、非同期処理終了後に宣言する
            //
            //NotifyComplete();
        }

                public void Refresh()
        {
            WebClient cli = new WebClient();
            Uri u = new Uri("http://www3.nhk.or.jp/rss/news/cat0.xml");
            cli.DownloadStringCompleted +=new DownloadStringCompletedEventHandler(cli_DownloadStringCompleted);
            cli.DownloadStringAsync(u);
        }

        void cli_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                //RSSの読み込みとXMLへの変換
                StringReader sr = new StringReader(e.Result);
                XmlReader xr = XmlReader.Create(sr);
                SyndicationFeed sf = SyndicationFeed.Load(xr);

                //最初の記事のタイトルと日時を取得
                SyndicationItem firstnews = sf.Items.First();
                String content = firstnews.Title.Text;
                String date = firstnews.PublishDate.ToLocalTime().ToString("M/d/yy h:mm:ss");

                //取得した情報をもとにタイルを更新
                UpdateTile(content, date);
            }
            NotifyComplete();
        }

        public static void UpdateTile(String content, String date)
        {
            //タイルがあるかどうかチェックしある場合は更新する
            ShellTile firsttile = ShellTile.ActiveTiles.First();
            if (firsttile != null)
            {
                //タイルの作成
                StandardTileData tile = new StandardTileData()
                {
                    Title = DateTime.Now.ToString("M/d h:mm 更新"),
                    BackTitle = date,　　//最新記事の日付
                    BackContent = content //最新
                };

                //作成したタイルで更新
                firsttile.Update(tile);
            }
        }
    }
}